from django import forms
from django.utils.translation import ugettext_lazy as _
from .models import Forum, Post, PostAnswer

class ForumForm(forms.ModelForm):

	class Meta:
		model = Forum
		fields = ('name', 'limit_date', 'description', 'topic', )
		labels = {
			'name': _('Title'),
			'description': _('Description'),
			'limit_date': _('Limit Date'),
		}
		help_texts = {
			'name': _('Forum title'),
			'description': _('What is this forum about?'),
			'limit_date': _('Limit date for students post on this forum'),
		}
		widgets = {
			'name': forms.TextInput(attrs={'class': 'form-control'}),
			'description': forms.Textarea(attrs={'cols': 80, 'rows': 5, 'class': 'form-control'}),
			'topic': forms.HiddenInput(),
			'limit_date': forms.DateInput(attrs={'class': 'date-picker form-control'}),
		}

class PostForm(forms.ModelForm):
	
	class Meta:
		model = Post
		fields = ('message', 'forum', )
		labels = {
			'message': _('Message')
		}
		widgets = {
			'message': forms.Textarea(attrs={'cols': 80, 'rows': 3}),
			'forum': forms.HiddenInput(),
		}

class PostAnswerForm(forms.ModelForm):

	class Meta:
		model = PostAnswer
		fields = ('message', 'post', )
		labels = {
			'message': _('Message')
		}
		widgets = {
			'message': forms.Textarea(attrs={'cols': 80, 'rows': 3}),
			'post': forms.HiddenInput(),
		}